#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<avr/pgmspace.h>
#include 	<util/delay.h>

#include 	"lcd.h"

//---------------------------------------------------------------------------------//
void lcd_strobe_E(void)
{
    // strobe E signal 
    LCD_PORT &= ~_BV(LCD_ENABLE_PIN);
    _delay_us(10); 
    LCD_PORT |= _BV(LCD_ENABLE_PIN);
	_delay_us(10); 
}
//---------------------------------------------------------------------------------//
void lcd_send_cmd(void)
{
    //is a command, so RS must be low (0) 
    LCD_PORT &= ~_BV(LCD_RS_PIN); 
    lcd_strobe_E();
    _delay_us(10);
}
//---------------------------------------------------------------------------------//
void lcd_send_char(void)
{
    //Character, so RS must be high (1) 
    LCD_PORT |= _BV(LCD_RS_PIN); 
    lcd_strobe_E();
    _delay_us(10);
}
//---------------------------------------------------------------------------------//
void lcd_load_byte (uint8_t dat)
{
  
    int i;
	
	LCD_PORT |= _BV(LCD_CLOCK_PIN);
	  
    for(i = 0; i <= 8; i++)
    {
        if (dat & 0x80)
			LCD_PORT |=  _BV(LCD_RS_PIN); 
        else
			LCD_PORT &= ~_BV(LCD_RS_PIN);
        
		dat = dat << 1;
		_delay_us (10);
        LCD_PORT &= ~_BV(LCD_CLOCK_PIN);
		_delay_us (10);
        LCD_PORT |= _BV(LCD_CLOCK_PIN);
		_delay_us (10);
    }
}
//---------------------------------------------------------------------------------//
void LCD_Write_cmd (uint8_t cmd)
{
	lcd_load_byte (cmd);
	lcd_send_cmd ();
}
//---------------------------------------------------------------------------------//
void LCD_Write_data (uint8_t cmd)
{
	lcd_load_byte (cmd);
	lcd_send_char ();
}
//---------------------------------------------------------------------------------//
void lcd_gotoxy (uint8_t x, uint8_t y)
{
	switch (y)
	{
		case 1 : 	LCD_Write_cmd (0x80 + x -1); 
					break;
		case 2 : 	LCD_Write_cmd (0xC0 + x -1); 
					break;
					
		default : break;
	}
}
//---------------------------------------------------------------------------------//
void lcd_putc (uint8_t c)
{
	switch (c)
	{
		
		case '\n'   : 	lcd_gotoxy (2,1);
						break;
		case '\b'	:	LCD_Write_cmd (0x10);
						break;
		default  	:	LCD_Write_data (c);
						break;
	}
}
//---------------------------------------------------------------------------------//
void LCD_DisplayCharacter (char dat)
{
	LCD_Write_data (dat);
}
//---------------------------------------------------------------------------------//
void lcd_printf (uint8_t *string,char nAligh)
{
	char cLine[STRDSPLMAX+1];
	int  len, i;
	
	memset (cLine, ' ', sizeof cLine);
	
	len = strlen (string);
	if (len > 16)
	    len = 16;
		
		switch (nAligh)
		{
            //OK
        case  DSPL_LEFT :  	memcpy (cLine, string, len);	 
							break;
           // OK
        case  DSPL_CENTER:  memcpy (cLine+((STRDSPLMAX - len)/2), string, len);
							break;
           // OK
        case  DSPL_RIGHT:   memcpy (cLine+ (STRDSPLMAX - len), string, len);
                            break;

               default : 	break;
		}	   
			   
		for (i = 0; i < STRDSPLMAX; i++)
		{
			lcd_putc (cLine[i]);
		}
}
//---------------------------------------------------------------------------------//
void lcd_clear (uint8_t line)
{
   if (line == 1)
      lcd_gotoxy (1,1);
   else
      if (line == 2)
         lcd_gotoxy (2,1);

   lcd_printf("                ",DSPL_LEFT);
}
//---------------------------------------------------------------------------------//
void init_lcd (void)
{
	
	_delay_ms(10);
	
	LCD_PORT |= _BV(LCD_ENABLE_PIN);
	LCD_PORT |= _BV(LCD_CLOCK_PIN);
		
	LCD_Write_cmd (LCD_FUNCTION_SET);	//lcd 5*7 Display
	LCD_Write_cmd (LCD_ON);				//Diaplay ON and Currsor ON
	LCD_Write_cmd (LCD_MODE_SET);		//mode set
	LCD_Write_cmd (LCD_CLEAR);			//clear
	
	_delay_us (10);
}
//-----------------------------------------------------------------------------------------//

